@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,18 +17,15 @@
 
 class BoardsController < ApplicationController
   default_search_scope :messages
-  before_filter :find_project, :find_board_if_available, :authorize
+  before_filter :find_project_by_project_id, :find_board_if_available, :authorize
   accept_rss_auth :index, :show
 
-  helper :messages
-  include MessagesHelper
   helper :sort
   include SortHelper
   helper :watchers
-  include WatchersHelper
 
   def index
-    @boards = @project.boards
+    @boards = @project.boards.includes(:last_message => :author).all
     # show the board if there is only one
     if @boards.size == 1
       @board = @boards.first
@@ -46,11 +43,11 @@ class BoardsController < ApplicationController
 
         @topic_count = @board.topics.count
         @topic_pages = Paginator.new self, @topic_count, per_page_option, params['page']
-        @topics =  @board.topics.find :all, :order => ["#{Message.table_name}.sticky DESC", sort_clause].compact.join(', '),
+        @topics =  @board.topics.reorder("#{Message.table_name}.sticky DESC").order(sort_clause).all(
                                       :include => [:author, {:last_reply => :author}],
                                       :limit  =>  @topic_pages.items_per_page,
-                                      :offset =>  @topic_pages.current.offset
-        @message = Message.new
+                                      :offset =>  @topic_pages.current.offset)
+        @message = Message.new(:board => @board)
         render :action => 'show', :layout => !request.xhr?
       }
       format.atom {
@@ -62,20 +59,31 @@ class BoardsController < ApplicationController
     end
   end
 
-  verify :method => :post, :only => [ :destroy ], :redirect_to => { :action => :index }
-
   def new
-    @board = Board.new(params[:board])
-    @board.project = @project
-    if request.post? && @board.save
+    @board = @project.boards.build
+    @board.safe_attributes = params[:board]
+  end
+
+  def create
+    @board = @project.boards.build
+    @board.safe_attributes = params[:board]
+    if @board.save
       flash[:notice] = l(:notice_successful_create)
       redirect_to_settings_in_projects
+    else
+      render :action => 'new'
     end
   end
 
   def edit
-    if request.post? && @board.update_attributes(params[:board])
+  end
+
+  def update
+    @board.safe_attributes = params[:board]
+    if @board.save
       redirect_to_settings_in_projects
+    else
+      render :action => 'edit'
     end
   end
 
@@ -89,12 +97,6 @@ private
     redirect_to :controller => 'projects', :action => 'settings', :id => @project, :tab => 'boards'
   end
 
-  def find_project
-    @project = Project.find(params[:project_id])
-  rescue ActiveRecord::RecordNotFound
-    render_404
-  end
-
   def find_board_if_available
     @board = @project.boards.find(params[:id]) if params[:id]
   rescue ActiveRecord::RecordNotFound
