@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@ class IssueCategoriesController < ApplicationController
   before_filter :find_project_by_project_id, :only => [:index, :new, :create]
   before_filter :authorize
   accept_api_auth :index, :show, :create, :update, :destroy
-  
+
   def index
     respond_to do |format|
       format.html { redirect_to :controller => 'projects', :action => 'settings', :tab => 'categories', :id => @project }
@@ -41,6 +41,11 @@ class IssueCategoriesController < ApplicationController
   def new
     @category = @project.issue_categories.build
     @category.safe_attributes = params[:issue_category]
+
+    respond_to do |format|
+      format.html
+      format.js
+    end
   end
 
   def create
@@ -52,20 +57,13 @@ class IssueCategoriesController < ApplicationController
           flash[:notice] = l(:notice_successful_create)
           redirect_to :controller => 'projects', :action => 'settings', :tab => 'categories', :id => @project
         end
-        format.js do
-          # IE doesn't support the replace_html rjs method for select box options
-          render(:update) {|page| page.replace "issue_category_id",
-            content_tag('select', content_tag('option') + options_from_collection_for_select(@project.issue_categories, 'id', 'name', @category.id), :id => 'issue_category_id', :name => 'issue[category_id]')
-          }
-        end
+        format.js
         format.api { render :action => 'show', :status => :created, :location => issue_category_path(@category) }
       end
     else
       respond_to do |format|
         format.html { render :action => 'new'}
-        format.js do
-          render(:update) {|page| page.alert(@category.errors.full_messages.join('\n')) }
-        end
+        format.js   { render :action => 'new'}
         format.api { render_validation_errors(@category) }
       end
     end
@@ -82,7 +80,7 @@ class IssueCategoriesController < ApplicationController
           flash[:notice] = l(:notice_successful_update)
           redirect_to :controller => 'projects', :action => 'settings', :tab => 'categories', :id => @project
         }
-        format.api { head :ok }
+        format.api { render_api_ok }
       end
     else
       respond_to do |format|
@@ -94,7 +92,7 @@ class IssueCategoriesController < ApplicationController
 
   def destroy
     @issue_count = @category.issues.size
-    if @issue_count == 0 || params[:todo] || api_request? 
+    if @issue_count == 0 || params[:todo] || api_request?
       reassign_to = nil
       if params[:reassign_to_id] && (params[:todo] == 'reassign' || params[:todo].blank?)
         reassign_to = @project.issue_categories.find_by_id(params[:reassign_to_id])
@@ -102,7 +100,7 @@ class IssueCategoriesController < ApplicationController
       @category.destroy(reassign_to)
       respond_to do |format|
         format.html { redirect_to :controller => 'projects', :action => 'settings', :id => @project, :tab => 'categories' }
-        format.api { head :ok }
+        format.api { render_api_ok }
       end
       return
     end
