@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -63,31 +63,16 @@ class MembersController < ApplicationController
     end
 
     respond_to do |format|
-      if members.present? && members.all? {|m| m.valid? }
-        format.html { redirect_to :controller => 'projects', :action => 'settings', :tab => 'members', :id => @project }
-        format.js {
-          render(:update) {|page|
-            page.replace_html "tab-content-members", :partial => 'projects/settings/members'
-            page << 'hideOnLoad()'
-            members.each {|member| page.visual_effect(:highlight, "member-#{member.id}") }
-          }
-        }
-        format.api {
-          @member = members.first
+      format.html { redirect_to :controller => 'projects', :action => 'settings', :tab => 'members', :id => @project }
+      format.js { @members = members }
+      format.api {
+        @member = members.first
+        if @member.valid?
           render :action => 'show', :status => :created, :location => membership_url(@member)
-        }
-      else
-        format.js {
-          render(:update) {|page|
-            errors = members.collect {|m|
-              m.errors.full_messages
-            }.flatten.uniq
-
-            page.alert(l(:notice_failed_to_save_members, :errors => errors.join(', ')))
-          }
-        }
-        format.api { render_validation_errors(members.first) }
-      end
+        else
+          render_validation_errors(@member)
+        end
+      }
     end
   end
 
@@ -98,16 +83,10 @@ class MembersController < ApplicationController
     saved = @member.save
     respond_to do |format|
       format.html { redirect_to :controller => 'projects', :action => 'settings', :tab => 'members', :id => @project }
-      format.js {
-        render(:update) {|page|
-          page.replace_html "tab-content-members", :partial => 'projects/settings/members'
-          page << 'hideOnLoad()'
-          page.visual_effect(:highlight, "member-#{@member.id}")
-        }
-      }
+      format.js
       format.api {
         if saved
-          head :ok
+          render_api_ok
         else
           render_validation_errors(@member)
         end
@@ -121,14 +100,10 @@ class MembersController < ApplicationController
     end
     respond_to do |format|
       format.html { redirect_to :controller => 'projects', :action => 'settings', :tab => 'members', :id => @project }
-      format.js { render(:update) {|page|
-          page.replace_html "tab-content-members", :partial => 'projects/settings/members'
-          page << 'hideOnLoad()'
-        }
-      }
+      format.js
       format.api {
         if @member.destroyed?
-          head :ok
+          render_api_ok
         else
           head :unprocessable_entity
         end
@@ -140,5 +115,4 @@ class MembersController < ApplicationController
     @principals = Principal.active.not_member_of(@project).like(params[:q]).all(:limit => 100)
     render :layout => false
   end
-
 end
