@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,7 +20,7 @@ class VersionsController < ApplicationController
   model_object Version
   before_filter :find_model_object, :except => [:index, :new, :create, :close_completed]
   before_filter :find_project_from_association, :except => [:index, :new, :create, :close_completed]
-  before_filter :find_project, :only => [:index, :new, :create, :close_completed]
+  before_filter :find_project_by_project_id, :only => [:index, :new, :create, :close_completed]
   before_filter :authorize
 
   accept_api_auth :index, :show, :create, :update, :destroy
@@ -78,13 +78,7 @@ class VersionsController < ApplicationController
 
     respond_to do |format|
       format.html
-      format.js do
-        render :update do |page|
-          page.replace_html 'ajax-modal', :partial => 'versions/new_modal'
-          page << "showModal('ajax-modal', '600px');"
-          page << "Form.Element.focus('version_name');"
-        end
-      end
+      format.js
     end
   end
 
@@ -103,14 +97,7 @@ class VersionsController < ApplicationController
             flash[:notice] = l(:notice_successful_create)
             redirect_back_or_default :controller => 'projects', :action => 'settings', :tab => 'versions', :id => @project
           end
-          format.js do
-            render(:update) {|page|
-              page << 'hideModal();'
-              # IE doesn't support the replace_html rjs method for select box options
-              page.replace "issue_fixed_version_id",
-                content_tag('select', content_tag('option') + version_options_for_select(@project.shared_versions.open, @version), :id => 'issue_fixed_version_id', :name => 'issue[fixed_version_id]')
-            }
-          end
+          format.js
           format.api do
             render :action => 'show', :status => :created, :location => version_url(@version)
           end
@@ -118,12 +105,7 @@ class VersionsController < ApplicationController
       else
         respond_to do |format|
           format.html { render :action => 'new' }
-          format.js do
-            render :update do |page|
-              page.replace_html 'ajax-modal', :partial => 'versions/new_modal'
-              page << "Form.Element.focus('version_name');"
-            end
-          end
+          format.js   { render :action => 'new' }
           format.api  { render_validation_errors(@version) }
         end
       end
@@ -144,7 +126,7 @@ class VersionsController < ApplicationController
             flash[:notice] = l(:notice_successful_update)
             redirect_back_or_default :controller => 'projects', :action => 'settings', :tab => 'versions', :id => @project
           }
-          format.api  { head :ok }
+          format.api  { render_api_ok }
         end
       else
         respond_to do |format|
@@ -167,7 +149,7 @@ class VersionsController < ApplicationController
       @version.destroy
       respond_to do |format|
         format.html { redirect_back_or_default :controller => 'projects', :action => 'settings', :tab => 'versions', :id => @project }
-        format.api  { head :ok }
+        format.api  { render_api_ok }
       end
     else
       respond_to do |format|
@@ -183,16 +165,11 @@ class VersionsController < ApplicationController
   def status_by
     respond_to do |format|
       format.html { render :action => 'show' }
-      format.js { render(:update) {|page| page.replace_html 'status_by', render_issue_status_by(@version, params[:status_by])} }
+      format.js
     end
   end
 
-private
-  def find_project
-    @project = Project.find(params[:project_id])
-  rescue ActiveRecord::RecordNotFound
-    render_404
-  end
+  private
 
   def retrieve_selected_tracker_ids(selectable_trackers, default_trackers=nil)
     if ids = params[:tracker_ids]
@@ -201,5 +178,4 @@ private
       @selected_tracker_ids = (default_trackers || selectable_trackers).collect {|t| t.id.to_s }
     end
   end
-
 end
