@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -33,15 +33,6 @@ class WatchersController < ApplicationController
   end
 
   def new
-    respond_to do |format|
-      format.js do
-        render :update do |page|
-          page.replace_html 'ajax-modal', :partial => 'watchers/new', :locals => {:watched => @watched}
-          page << "showModal('ajax-modal', '400px');"
-          page << "$('ajax-modal').addClassName('new-watcher');"
-        end
-      end
-    end
   end
 
   def create
@@ -52,34 +43,15 @@ class WatchersController < ApplicationController
       end
     end
     respond_to do |format|
-      format.html { redirect_to :back }
-      format.js do
-        render :update do |page|
-          page.replace_html 'ajax-modal', :partial => 'watchers/new', :locals => {:watched => @watched}
-          page.replace_html 'watchers', :partial => 'watchers/watchers', :locals => {:watched => @watched}
-        end
-      end
+      format.html { redirect_to_referer_or {render :text => 'Watcher added.', :layout => true}}
+      format.js
     end
-  rescue ::ActionController::RedirectBackError
-    render :text => 'Watcher added.', :layout => true
   end
 
   def append
     if params[:watcher].is_a?(Hash)
       user_ids = params[:watcher][:user_ids] || [params[:watcher][:user_id]]
-      users = User.active.find_all_by_id(user_ids)
-      respond_to do |format|
-        format.js do
-          render :update do |page|
-            users.each do |user|
-              page.select("#issue_watcher_user_ids_#{user.id}").each do |item|
-                page.remove item
-              end
-            end
-            page.insert_html :bottom, 'watchers_inputs', :text => watchers_checkboxes(nil, users, true)
-          end
-        end
-      end
+      @users = User.active.find_all_by_id(user_ids)
     end
   end
 
@@ -87,11 +59,7 @@ class WatchersController < ApplicationController
     @watched.set_watcher(User.find(params[:user_id]), false) if request.post?
     respond_to do |format|
       format.html { redirect_to :back }
-      format.js do
-        render :update do |page|
-          page.replace_html 'watchers', :partial => 'watchers/watchers', :locals => {:watched => @watched}
-        end
-      end
+      format.js
     end
   end
 
@@ -111,7 +79,7 @@ private
       @watched = klass.find(params[:object_id])
       @project = @watched.project
     elsif params[:project_id]
-      @project = Project.visible.find(params[:project_id])
+      @project = Project.visible.find_by_param(params[:project_id])
     end
   rescue
     render_404
@@ -120,17 +88,8 @@ private
   def set_watcher(user, watching)
     @watched.set_watcher(user, watching)
     respond_to do |format|
-      format.html { redirect_to :back }
-      format.js do
-        render(:update) do |page|
-          c = watcher_css(@watched)
-          page.select(".#{c}").each do |item|
-            page.replace_html item, watcher_link(@watched, user)
-          end
-        end
-      end
+      format.html { redirect_to_referer_or {render :text => (watching ? 'Watcher added.' : 'Watcher removed.'), :layout => true}}
+      format.js { render :partial => 'set_watcher', :locals => {:user => user, :watched => @watched} }
     end
-  rescue ::ActionController::RedirectBackError
-    render :text => (watching ? 'Watcher added.' : 'Watcher removed.'), :layout => true
   end
 end
