@@ -1,7 +1,7 @@
 # encoding: utf-8
 #
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,14 +21,14 @@ module WikiHelper
 
   def wiki_page_options_for_select(pages, selected = nil, parent = nil, level = 0)
     pages = pages.group_by(&:parent) unless pages.is_a?(Hash)
-    s = ''
+    s = ''.html_safe
     if pages.has_key?(parent)
       pages[parent].each do |page|
         attrs = "value='#{page.id}'"
         attrs << " selected='selected'" if selected == page
-        indent = (level > 0) ? ('&nbsp;' * level * 2 + '&#187; ') : nil
+        indent = (level > 0) ? ('&nbsp;' * level * 2 + '&#187; ') : ''
 
-        s << "<option #{attrs}>#{indent}#{h page.pretty_title}</option>\n" +
+        s << content_tag('option', (indent + h(page.pretty_title)).html_safe, :value => page.id.to_s, :selected => selected == page) +
                wiki_page_options_for_select(pages, selected, page, level + 1)
       end
     end
@@ -37,7 +37,7 @@ module WikiHelper
 
   def wiki_page_breadcrumb(page)
     breadcrumb(page.ancestors.reverse.collect {|parent|
-      link_to(h(parent.pretty_title), {:controller => 'wiki', :action => 'show', :id => parent.title, :project_id => parent.project})
+      link_to(h(parent.pretty_title), {:controller => 'wiki', :action => 'show', :id => parent.title, :project_id => parent.project, :version => nil})
     })
   end
 end
