@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,4 +31,10 @@ class DocumentCategory < Enumeration
   def transfer_relations(to)
     documents.update_all("category_id = #{to.id}")
   end
+
+  def self.default
+    d = super
+    d = first if d.nil?
+    d
+  end
 end
