@@ -505,7 +505,7 @@ class RedCloth3 < String
         atts
     end
 
-    STYLES_RE = /^(color|width|height|border|background|padding|margin|font|text)(-[a-z]+)*:\s*((\d+%?|\d+px|\d+(\.\d+)?em|#[0-9a-f]+|[a-z]+)\s*)+$/i
+    STYLES_RE = /^(color|width|height|border|background|padding|margin|font|text|float)(-[a-z]+)*:\s*((\d+%?|\d+px|\d+(\.\d+)?em|#[0-9a-f]+|[a-z]+)\s*)+$/i
 
     def sanitize_styles(str)
       styles = str.split(";").map(&:strip)
@@ -816,10 +816,10 @@ class RedCloth3 < String
             ":
             (                          # $url
             (\/|[a-zA-Z]+:\/\/|www\.|mailto:)  # $proto
-            [\w\/]\S+?
+            [[:alnum:]_\/]\S+?
             )               
             (\/)?                      # $slash
-            ([^\w\=\/;\(\)]*?)         # $post
+            ([^[:alnum:]_\=\/;\(\)]*?)         # $post
             )
             (?=<|\s|$)
         /x 
