@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -55,6 +55,20 @@ class BoardsControllerTest < ActionController::TestCase
     assert_not_nil assigns(:topics)
   end
 
+  def test_show_should_display_sticky_messages_first
+    Message.update_all(:sticky => 0)
+    Message.update_all({:sticky => 1}, {:id => 1})
+
+    get :show, :project_id => 1, :id => 1
+    assert_response :success
+
+    topics = assigns(:topics)
+    assert_not_nil topics
+    assert topics.size > 1, "topics size was #{topics.size}"
+    assert topics.first.sticky?
+    assert topics.first.updated_on < topics.second.updated_on
+  end
+
   def test_show_with_permission_should_display_the_new_message_form
     @request.session[:user_id] = 2
     get :show, :project_id => 1, :id => 1
@@ -84,6 +98,23 @@ class BoardsControllerTest < ActionController::TestCase
     get :new, :project_id => 1
     assert_response :success
     assert_template 'new'
+
+    assert_select 'select[name=?]', 'board[parent_id]' do
+      assert_select 'option', (Project.find(1).boards.size + 1)
+      assert_select 'option[value=]', :text => ''
+      assert_select 'option[value=1]', :text => 'Help'
+    end
+  end
+
+  def test_new_without_project_boards
+    Project.find(1).boards.delete_all
+    @request.session[:user_id] = 2
+
+    get :new, :project_id => 1
+    assert_response :success
+    assert_template 'new'
+
+    assert_select 'select[name=?]', 'board[parent_id]', 0
   end
 
   def test_create
@@ -97,6 +128,16 @@ class BoardsControllerTest < ActionController::TestCase
     assert_equal 'Testing board creation', board.description
   end
 
+  def test_create_with_parent
+    @request.session[:user_id] = 2
+    assert_difference 'Board.count' do
+      post :create, :project_id => 1, :board => { :name => 'Testing', :description => 'Testing', :parent_id => 2}
+    end
+    assert_redirected_to '/projects/ecookbook/settings/boards'
+    board = Board.first(:order => 'id DESC')
+    assert_equal Board.find(2), board.parent
+  end
+
   def test_create_with_failure
     @request.session[:user_id] = 2
     assert_no_difference 'Board.count' do
@@ -113,6 +154,18 @@ class BoardsControllerTest < ActionController::TestCase
     assert_template 'edit'
   end
 
+  def test_edit_with_parent
+    board = Board.generate!(:project_id => 1, :parent_id => 2)
+    @request.session[:user_id] = 2
+    get :edit, :project_id => 1, :id => board.id
+    assert_response :success
+    assert_template 'edit'
+
+    assert_select 'select[name=?]', 'board[parent_id]' do
+      assert_select 'option[value=2][selected=selected]'
+    end
+  end
+
   def test_update
     @request.session[:user_id] = 2
     assert_no_difference 'Board.count' do
