@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,6 +30,12 @@ class EnumerationsControllerTest < ActionController::TestCase
     assert_template 'index'
   end
 
+  def test_index_should_require_admin
+    @request.session[:user_id] = nil
+    get :index
+    assert_response 302
+  end
+
   def test_new
     get :new, :type => 'IssuePriority'
     assert_response :success
@@ -48,7 +54,7 @@ class EnumerationsControllerTest < ActionController::TestCase
     assert_difference 'IssuePriority.count' do
       post :create, :enumeration => {:type => 'IssuePriority', :name => 'Lowest'}
     end
-    assert_redirected_to '/enumerations?type=IssuePriority'
+    assert_redirected_to '/enumerations'
     e = IssuePriority.find_by_name('Lowest')
     assert_not_nil e
   end
@@ -77,7 +83,7 @@ class EnumerationsControllerTest < ActionController::TestCase
     assert_no_difference 'IssuePriority.count' do
       put :update, :id => 6, :enumeration => {:type => 'IssuePriority', :name => 'New name'}
     end
-    assert_redirected_to '/enumerations?type=IssuePriority'
+    assert_redirected_to '/enumerations'
     e = IssuePriority.find(6)
     assert_equal 'New name', e.name
   end
@@ -105,6 +111,9 @@ class EnumerationsControllerTest < ActionController::TestCase
     assert_response :success
     assert_template 'destroy'
     assert_not_nil Enumeration.find_by_id(4)
+    assert_select 'select[name=reassign_to_id]' do
+      assert_select 'option[value=6]', :text => 'High'
+    end
   end
 
   def test_destroy_enumeration_in_use_with_reassignment
