@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,7 +22,8 @@ require 'issue_categories_controller'
 class IssueCategoriesController; def rescue_action(e) raise e end; end
 
 class IssueCategoriesControllerTest < ActionController::TestCase
-  fixtures :projects, :users, :members, :member_roles, :roles, :enabled_modules, :issue_categories
+  fixtures :projects, :users, :members, :member_roles, :roles, :enabled_modules, :issue_categories,
+           :issues
 
   def setup
     @controller = IssueCategoriesController.new
@@ -37,6 +38,16 @@ class IssueCategoriesControllerTest < ActionController::TestCase
     get :new, :project_id => '1'
     assert_response :success
     assert_template 'new'
+    assert_select 'input[name=?]', 'issue_category[name]'
+  end
+
+  def test_new_from_issue_form
+    @request.session[:user_id] = 2 # manager
+    xhr :get, :new, :project_id => '1'
+
+    assert_response :success
+    assert_template 'new'
+    assert_equal 'text/javascript', response.content_type
   end
 
   def test_create
@@ -66,9 +77,8 @@ class IssueCategoriesControllerTest < ActionController::TestCase
     assert_equal 'New category', category.name
 
     assert_response :success
-    assert_select_rjs :replace, 'issue_category_id' do
-      assert_select "option[value=#{category.id}][selected=selected]"
-    end
+    assert_template 'create'
+    assert_equal 'text/javascript', response.content_type
   end
 
   def test_create_from_issue_form_with_failure
@@ -78,7 +88,8 @@ class IssueCategoriesControllerTest < ActionController::TestCase
     end
 
     assert_response :success
-    assert_match /alert/, @response.body
+    assert_template 'new'
+    assert_equal 'text/javascript', response.content_type
   end
 
   def test_edit
@@ -86,6 +97,7 @@ class IssueCategoriesControllerTest < ActionController::TestCase
     get :edit, :id => 2
     assert_response :success
     assert_template 'edit'
+    assert_select 'input[name=?][value=?]', 'issue_category[name]', 'Recipes'
   end
 
   def test_update
