@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -40,7 +40,7 @@ class RepositoriesControllerTest < ActionController::TestCase
     assert_template 'new'
     assert_kind_of Repository::Subversion, assigns(:repository)
     assert assigns(:repository).new_record?
-    assert_tag 'input', :attributes => {:name => 'repository[url]'}
+    assert_tag 'input', :attributes => {:name => 'repository[url]', :disabled => nil}
   end
 
   def test_new_should_propose_enabled_scm_only
@@ -91,7 +91,7 @@ class RepositoriesControllerTest < ActionController::TestCase
     assert_response :success
     assert_template 'edit'
     assert_equal Repository.find(11), assigns(:repository)
-    assert_tag 'input', :attributes => {:name => 'repository[url]', :value => 'svn://localhost/test'}
+    assert_tag 'input', :attributes => {:name => 'repository[url]', :value => 'svn://localhost/test', :disabled => 'disabled'}
   end
 
   def test_update
@@ -171,20 +171,25 @@ class RepositoriesControllerTest < ActionController::TestCase
   def test_add_related_issue
     @request.session[:user_id] = 2
     assert_difference 'Changeset.find(103).issues.size' do
-      post :add_related_issue, :id => 1, :rev => 4, :issue_id => 2, :format => 'js'
+      xhr :post, :add_related_issue, :id => 1, :rev => 4, :issue_id => 2, :format => 'js'
       assert_response :success
+      assert_template 'add_related_issue'
+      assert_equal 'text/javascript', response.content_type
     end
-    assert_select_rjs :replace_html, 'related-issues'
     assert_equal [2], Changeset.find(103).issue_ids
+    assert_include 'related-issues', response.body
+    assert_include 'Feature request #2', response.body
   end
 
   def test_add_related_issue_with_invalid_issue_id
     @request.session[:user_id] = 2
     assert_no_difference 'Changeset.find(103).issues.size' do
-      post :add_related_issue, :id => 1, :rev => 4, :issue_id => 9999, :format => 'js'
+      xhr :post, :add_related_issue, :id => 1, :rev => 4, :issue_id => 9999, :format => 'js'
       assert_response :success
+      assert_template 'add_related_issue'
+      assert_equal 'text/javascript', response.content_type
     end
-    assert_include 'alert("Issue is invalid")', @response.body
+    assert_include 'alert("Issue is invalid")', response.body
   end
 
   def test_remove_related_issue
@@ -193,14 +198,21 @@ class RepositoriesControllerTest < ActionController::TestCase
 
     @request.session[:user_id] = 2
     assert_difference 'Changeset.find(103).issues.size', -1 do
-      delete :remove_related_issue, :id => 1, :rev => 4, :issue_id => 2, :format => 'js'
+      xhr :delete, :remove_related_issue, :id => 1, :rev => 4, :issue_id => 2, :format => 'js'
       assert_response :success
+      assert_template 'remove_related_issue'
+      assert_equal 'text/javascript', response.content_type
     end
-    assert_select_rjs :remove, 'related-issue-2'
     assert_equal [1], Changeset.find(103).issue_ids
+    assert_include 'related-issue-2', response.body
   end
 
   def test_graph_commits_per_month
+    # Make sure there's some data to display
+    latest = Project.find(1).repository.changesets.maximum(:commit_date)
+    assert_not_nil latest
+    Date.stubs(:today).returns(latest.to_date + 10)
+
     get :graph, :id => 1, :graph => 'commits_per_month'
     assert_response :success
     assert_equal 'image/svg+xml', @response.content_type
