@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -33,6 +33,10 @@ class SysControllerTest < ActionController::TestCase
     Setting.enabled_scm = %w(Subversion Git)
   end
 
+  def teardown
+    Setting.clear_cache
+  end
+
   def test_projects_with_repository_enabled
     get :projects
     assert_response :success
