@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,24 +36,22 @@ class TimelogControllerTest < ActionController::TestCase
     @response   = ActionController::TestResponse.new
   end
 
-  def test_get_new
+  def test_new_with_project_id
     @request.session[:user_id] = 3
     get :new, :project_id => 1
     assert_response :success
     assert_template 'new'
-    # Default activity selected
-    assert_tag :tag => 'option', :attributes => { :selected => 'selected' },
-                                 :content => 'Development'
-    assert_select 'input[name=project_id][value=1]'
+    assert_select 'select[name=?]', 'time_entry[project_id]', 0
+    assert_select 'input[name=?][value=1][type=hidden]', 'time_entry[project_id]'
   end
 
-  def test_get_new_should_only_show_active_time_entry_activities
+  def test_new_with_issue_id
     @request.session[:user_id] = 3
-    get :new, :project_id => 1
+    get :new, :issue_id => 2
     assert_response :success
     assert_template 'new'
-    assert_no_tag 'select', :attributes => {:name => 'time_entry[project_id]'}
-    assert_no_tag 'option', :content => 'Inactive Activity'
+    assert_select 'select[name=?]', 'time_entry[project_id]', 0
+    assert_select 'input[name=?][value=1][type=hidden]', 'time_entry[project_id]'
   end
 
   def test_new_without_project
@@ -61,8 +59,8 @@ class TimelogControllerTest < ActionController::TestCase
     get :new
     assert_response :success
     assert_template 'new'
-    assert_tag 'select', :attributes => {:name => 'time_entry[project_id]'}
-    assert_select 'input[name=project_id]', 0
+    assert_select 'select[name=?]', 'time_entry[project_id]'
+    assert_select 'input[name=?]', 'time_entry[project_id]', 0
   end
 
   def test_new_without_project_should_prefill_the_form
@@ -73,7 +71,7 @@ class TimelogControllerTest < ActionController::TestCase
     assert_select 'select[name=?]', 'time_entry[project_id]' do
       assert_select 'option[value=1][selected=selected]'
     end
-    assert_select 'input[name=project_id]', 0
+    assert_select 'input[name=?]', 'time_entry[project_id]', 0
   end
 
   def test_new_without_project_should_deny_without_permission
@@ -84,6 +82,22 @@ class TimelogControllerTest < ActionController::TestCase
     assert_response 403
   end
 
+  def test_new_should_select_default_activity
+    @request.session[:user_id] = 3
+    get :new, :project_id => 1
+    assert_response :success
+    assert_select 'select[name=?]', 'time_entry[activity_id]' do
+      assert_select 'option[selected=selected]', :text => 'Development'
+    end
+  end
+
+  def test_new_should_only_show_active_time_entry_activities
+    @request.session[:user_id] = 3
+    get :new, :project_id => 1
+    assert_response :success
+    assert_no_tag 'option', :content => 'Inactive Activity'
+  end
+
   def test_get_edit_existing_time
     @request.session[:user_id] = 2
     get :edit, :id => 2, :project_id => nil
@@ -291,6 +305,12 @@ class TimelogControllerTest < ActionController::TestCase
 
     # System wide custom field
     assert_tag :select, :attributes => {:name => 'time_entry[custom_field_values][10]'}
+
+    # Activities
+    assert_select 'select[name=?]', 'time_entry[activity_id]' do
+      assert_select 'option[value=]', :text => '(No change)'
+      assert_select 'option[value=9]', :text => 'Design'
+    end
   end
 
   def test_get_bulk_edit_on_different_projects
@@ -463,6 +483,18 @@ class TimelogControllerTest < ActionController::TestCase
       :attributes => {:action => "/projects/ecookbook/time_entries", :id => 'query_form'}
   end
 
+  def test_index_from_a_date
+    get :index, :project_id => 'ecookbook', :from => "2007-03-23", :to => ""
+    assert_equal '2007-03-23'.to_date, assigns(:from)
+    assert_nil assigns(:to)
+  end
+
+  def test_index_to_a_date
+    get :index, :project_id => 'ecookbook', :from => "", :to => "2007-03-23"
+    assert_nil assigns(:from)
+    assert_equal '2007-03-23'.to_date, assigns(:to)
+  end
+
   def test_index_today
     Date.stubs(:today).returns('2011-12-15'.to_date)
     get :index, :period => 'today'
@@ -486,15 +518,15 @@ class TimelogControllerTest < ActionController::TestCase
 
   def test_index_last_week
     Date.stubs(:today).returns('2011-12-15'.to_date)
-    get :index, :period => 'current_week'
+    get :index, :period => 'last_week'
     assert_equal '2011-12-05'.to_date, assigns(:from)
     assert_equal '2011-12-11'.to_date, assigns(:to)
   end
 
-  def test_index_last_week
+  def test_index_last_2_week
     Date.stubs(:today).returns('2011-12-15'.to_date)
-    get :index, :period => 'last_week'
-    assert_equal '2011-12-05'.to_date, assigns(:from)
+    get :index, :period => 'last_2_weeks'
+    assert_equal '2011-11-28'.to_date, assigns(:from)
     assert_equal '2011-12-11'.to_date, assigns(:to)
   end
 
@@ -576,7 +608,7 @@ class TimelogControllerTest < ActionController::TestCase
     Setting.date_format = '%m/%d/%Y'
     get :index, :format => 'csv'
     assert_response :success
-    assert_equal 'text/csv', @response.content_type
+    assert_equal 'text/csv; header=present', @response.content_type
     assert @response.body.include?("Date,User,Activity,Project,Issue,Tracker,Subject,Hours,Comment,Overtime\n")
     assert @response.body.include?("\n04/21/2007,redMine Admin,Design,eCookbook,3,Bug,Error 281 when updating a recipe,1.0,\"\",\"\"\n")
   end
@@ -585,7 +617,7 @@ class TimelogControllerTest < ActionController::TestCase
     Setting.date_format = '%m/%d/%Y'
     get :index, :project_id => 1, :format => 'csv'
     assert_response :success
-    assert_equal 'text/csv', @response.content_type
+    assert_equal 'text/csv; header=present', @response.content_type
     assert @response.body.include?("Date,User,Activity,Project,Issue,Tracker,Subject,Hours,Comment,Overtime\n")
     assert @response.body.include?("\n04/21/2007,redMine Admin,Design,eCookbook,3,Bug,Error 281 when updating a recipe,1.0,\"\",\"\"\n")
   end
@@ -631,7 +663,7 @@ class TimelogControllerTest < ActionController::TestCase
     get :index, :project_id => 1, :format => 'csv',
         :from => '2011-11-10', :to => '2011-11-10'
     assert_response :success
-    assert_equal 'text/csv', @response.content_type
+    assert_equal 'text/csv; header=present', @response.content_type
     ar = @response.body.chomp.split("\n")
     s1 = "\xa4\xe9\xb4\xc1"
     if str_utf8.respond_to?(:force_encoding)
@@ -668,7 +700,7 @@ class TimelogControllerTest < ActionController::TestCase
     get :index, :project_id => 1, :format => 'csv',
         :from => '2011-11-10', :to => '2011-11-10'
     assert_response :success
-    assert_equal 'text/csv', @response.content_type
+    assert_equal 'text/csv; header=present', @response.content_type
     ar = @response.body.chomp.split("\n")
     s1 = "\xa4\xe9\xb4\xc1"
     if str_utf8.respond_to?(:force_encoding)
@@ -705,7 +737,7 @@ class TimelogControllerTest < ActionController::TestCase
       get :index, :project_id => 1, :format => 'csv',
           :from => '2011-11-10', :to => '2011-11-10'
       assert_response :success
-      assert_equal 'text/csv', @response.content_type
+      assert_equal 'text/csv; header=present', @response.content_type
 
       ar = @response.body.chomp.split("\n")
       s2 = ar[1].split(",")[7]
@@ -739,7 +771,7 @@ class TimelogControllerTest < ActionController::TestCase
       get :index, :project_id => 1, :format => 'csv',
           :from => '2011-11-10', :to => '2011-11-10'
       assert_response :success
-      assert_equal 'text/csv', @response.content_type
+      assert_equal 'text/csv; header=present', @response.content_type
 
       ar = @response.body.chomp.split("\n")
       s2 = ar[1].split(";")[7]
