@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,7 +24,9 @@ class UsersController; def rescue_action(e) raise e end; end
 class UsersControllerTest < ActionController::TestCase
   include Redmine::I18n
 
-  fixtures :users, :projects, :members, :member_roles, :roles, :auth_sources, :custom_fields, :custom_values, :groups_users
+  fixtures :users, :projects, :members, :member_roles, :roles,
+           :custom_fields, :custom_values, :groups_users,
+           :auth_sources
 
   def setup
     @controller = UsersController.new
@@ -49,6 +51,14 @@ class UsersControllerTest < ActionController::TestCase
     assert_nil assigns(:users).detect {|u| !u.active?}
   end
 
+  def test_index_with_status_filter
+    get :index, :status => 3
+    assert_response :success
+    assert_template 'index'
+    assert_not_nil assigns(:users)
+    assert_equal [3], assigns(:users).map(&:status).uniq
+  end
+
   def test_index_with_name_filter
     get :index, :name => 'john'
     assert_response :success
@@ -66,6 +76,9 @@ class UsersControllerTest < ActionController::TestCase
     users = assigns(:users)
     assert users.any?
     assert_equal([], (users - Group.find(10).users))
+    assert_select 'select[name=group_id]' do
+      assert_select 'option[value=10][selected=selected]'
+    end
   end
 
   def test_show
@@ -145,7 +158,6 @@ class UsersControllerTest < ActionController::TestCase
 
   def test_new
     get :new
-
     assert_response :success
     assert_template :new
     assert assigns(:user)
@@ -161,8 +173,8 @@ class UsersControllerTest < ActionController::TestCase
             :firstname => 'John',
             :lastname => 'Doe',
             :login => 'jdoe',
-            :password => 'secret',
-            :password_confirmation => 'secret',
+            :password => 'secret123',
+            :password_confirmation => 'secret123',
             :mail => 'jdoe@gmail.com',
             :mail_notification => 'none'
           },
@@ -178,26 +190,51 @@ class UsersControllerTest < ActionController::TestCase
     assert_equal 'jdoe', user.login
     assert_equal 'jdoe@gmail.com', user.mail
     assert_equal 'none', user.mail_notification
-    assert user.check_password?('secret')
+    assert user.check_password?('secret123')
 
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
     assert_equal [user.mail], mail.bcc
-    assert mail.body.include?('secret')
+    assert_mail_body_match 'secret', mail
+  end
+
+  def test_create_with_preferences
+    assert_difference 'User.count' do
+      post :create,
+        :user => {
+          :firstname => 'John',
+          :lastname => 'Doe',
+          :login => 'jdoe',
+          :password => 'secret123',
+          :password_confirmation => 'secret123',
+          :mail => 'jdoe@gmail.com',
+          :mail_notification => 'none'
+        },
+        :pref => {
+          'hide_mail' => '1',
+          'time_zone' => 'Paris',
+          'comments_sorting' => 'desc',
+          'warn_on_leaving_unsaved' => '0'
+        }
+    end
+    user = User.first(:order => 'id DESC')
+    assert_equal 'jdoe', user.login
+    assert_equal true, user.pref.hide_mail
+    assert_equal 'Paris', user.pref.time_zone
+    assert_equal 'desc', user.pref[:comments_sorting]
+    assert_equal '0', user.pref[:warn_on_leaving_unsaved]
   end
 
   def test_create_with_failure
     assert_no_difference 'User.count' do
       post :create, :user => {}
     end
-
     assert_response :success
     assert_template 'new'
   end
 
   def test_edit
     get :edit, :id => 2
-
     assert_response :success
     assert_template 'edit'
     assert_equal User.find(2), assigns(:user)
@@ -205,8 +242,9 @@ class UsersControllerTest < ActionController::TestCase
 
   def test_update
     ActionMailer::Base.deliveries.clear
-    put :update, :id => 2, :user => {:firstname => 'Changed', :mail_notification => 'only_assigned'}, :pref => {:hide_mail => '1', :comments_sorting => 'desc'}
-
+    put :update, :id => 2,
+        :user => {:firstname => 'Changed', :mail_notification => 'only_assigned'},
+        :pref => {:hide_mail => '1', :comments_sorting => 'desc'}
     user = User.find(2)
     assert_equal 'Changed', user.firstname
     assert_equal 'only_assigned', user.mail_notification
@@ -219,14 +257,12 @@ class UsersControllerTest < ActionController::TestCase
     assert_no_difference 'User.count' do
       put :update, :id => 2, :user => {:firstname => ''}
     end
-
     assert_response :success
     assert_template 'edit'
   end
 
   def test_update_with_group_ids_should_assign_groups
     put :update, :id => 2, :user => {:group_ids => ['10']}
-
     user = User.find(2)
     assert_equal [10], user.group_ids
   end
@@ -244,33 +280,56 @@ class UsersControllerTest < ActionController::TestCase
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
     assert_equal ['foo.bar@somenet.foo'], mail.bcc
-    assert mail.body.include?(ll('fr', :notice_account_activated))
+    assert_mail_body_match ll('fr', :notice_account_activated), mail
   end
 
   def test_update_with_password_change_should_send_a_notification
     ActionMailer::Base.deliveries.clear
     Setting.bcc_recipients = '1'
 
-    put :update, :id => 2, :user => {:password => 'newpass', :password_confirmation => 'newpass'}, :send_information => '1'
+    put :update, :id => 2, :user => {:password => 'newpass123', :password_confirmation => 'newpass123'}, :send_information => '1'
     u = User.find(2)
-    assert u.check_password?('newpass')
+    assert u.check_password?('newpass123')
 
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
     assert_equal [u.mail], mail.bcc
-    assert mail.body.include?('newpass')
+    assert_mail_body_match 'newpass123', mail
   end
 
-  test "put :update with a password change to an AuthSource user switching to Internal authentication" do
+  def test_update_user_switchin_from_auth_source_to_password_authentication
     # Configure as auth source
     u = User.find(2)
     u.auth_source = AuthSource.find(1)
     u.save!
 
-    put :update, :id => u.id, :user => {:auth_source_id => '', :password => 'newpass'}, :password_confirmation => 'newpass'
+    put :update, :id => u.id, :user => {:auth_source_id => '', :password => 'newpass123', :password_confirmation => 'newpass123'}
 
     assert_equal nil, u.reload.auth_source
-    assert u.check_password?('newpass')
+    assert u.check_password?('newpass123')
+  end
+
+  def test_update_notified_project
+    get :edit, :id => 2
+    assert_response :success
+    assert_template 'edit'
+    u = User.find(2)
+    assert_equal [1, 2, 5], u.projects.collect{|p| p.id}.sort
+    assert_equal [1, 2, 5], u.notified_projects_ids.sort
+    assert_tag :tag => 'input',
+               :attributes => {
+                  :id    => 'notified_project_ids_',
+                  :value => 1,
+                }
+    assert_equal 'all', u.mail_notification
+    put :update, :id => 2,
+        :user => {
+           :mail_notification => 'selected',
+         },
+        :notified_project_ids => [1, 2]
+    u = User.find(2)
+    assert_equal 'selected', u.mail_notification
+    assert_equal [1, 2], u.notified_projects_ids.sort
   end
 
   def test_destroy
@@ -281,13 +340,6 @@ class UsersControllerTest < ActionController::TestCase
     assert_nil User.find_by_id(2)
   end
 
-  def test_destroy_should_not_accept_get_requests
-    assert_no_difference 'User.count' do
-      get :destroy, :id => 2
-    end
-    assert_response 405
-  end
-
   def test_destroy_should_be_denied_for_non_admin_users
     @request.session[:user_id] = 3
 
@@ -297,16 +349,84 @@ class UsersControllerTest < ActionController::TestCase
     assert_response 403
   end
 
-  def test_edit_membership
-    post :edit_membership, :id => 2, :membership_id => 1,
-                           :membership => { :role_ids => [2]}
-    assert_redirected_to :action => 'edit', :id => '2', :tab => 'memberships'
+  def test_destroy_should_redirect_to_back_url_param
+    assert_difference 'User.count', -1 do
+      delete :destroy, :id => 2, :back_url => '/users?name=foo'
+    end
+    assert_redirected_to '/users?name=foo'
+  end
+
+  def test_create_membership
+    assert_difference 'Member.count' do
+      post :edit_membership, :id => 7, :membership => { :project_id => 3, :role_ids => [2]}
+    end
+    assert_redirected_to :action => 'edit', :id => '7', :tab => 'memberships'
+    member = Member.first(:order => 'id DESC')
+    assert_equal User.find(7), member.principal
+    assert_equal [2], member.role_ids
+    assert_equal 3, member.project_id
+  end
+
+  def test_create_membership_js_format
+    assert_difference 'Member.count' do
+      post :edit_membership, :id => 7, :membership => {:project_id => 3, :role_ids => [2]}, :format => 'js'
+      assert_response :success
+      assert_template 'edit_membership'
+      assert_equal 'text/javascript', response.content_type
+    end
+    member = Member.first(:order => 'id DESC')
+    assert_equal User.find(7), member.principal
+    assert_equal [2], member.role_ids
+    assert_equal 3, member.project_id
+    assert_include 'tab-content-memberships', response.body
+  end
+
+  def test_create_membership_js_format_with_failure
+    assert_no_difference 'Member.count' do
+      post :edit_membership, :id => 7, :membership => {:project_id => 3}, :format => 'js'
+      assert_response :success
+      assert_template 'edit_membership'
+      assert_equal 'text/javascript', response.content_type
+    end
+    assert_include 'alert', response.body, "Alert message not sent"
+    assert_include 'Role can\\\'t be empty', response.body, "Error message not sent"
+  end
+
+  def test_update_membership
+    assert_no_difference 'Member.count' do
+      put :edit_membership, :id => 2, :membership_id => 1, :membership => { :role_ids => [2]}
+      assert_redirected_to :action => 'edit', :id => '2', :tab => 'memberships'
+    end
     assert_equal [2], Member.find(1).role_ids
   end
 
+  def test_update_membership_js_format
+    assert_no_difference 'Member.count' do
+      put :edit_membership, :id => 2, :membership_id => 1, :membership => {:role_ids => [2]}, :format => 'js'
+      assert_response :success
+      assert_template 'edit_membership'
+      assert_equal 'text/javascript', response.content_type
+    end
+    assert_equal [2], Member.find(1).role_ids
+    assert_include 'tab-content-memberships', response.body
+  end
+
   def test_destroy_membership
-    post :destroy_membership, :id => 2, :membership_id => 1
+    assert_difference 'Member.count', -1 do
+      delete :destroy_membership, :id => 2, :membership_id => 1
+    end
     assert_redirected_to :action => 'edit', :id => '2', :tab => 'memberships'
     assert_nil Member.find_by_id(1)
   end
+
+  def test_destroy_membership_js_format
+    assert_difference 'Member.count', -1 do
+      delete :destroy_membership, :id => 2, :membership_id => 1, :format => 'js'
+      assert_response :success
+      assert_template 'destroy_membership'
+      assert_equal 'text/javascript', response.content_type
+    end
+    assert_nil Member.find_by_id(1)
+    assert_include 'tab-content-memberships', response.body
+  end
 end
