@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -75,7 +75,7 @@ class ApiTest::IssueRelationsTest < ActionController::IntegrationTest
           end
 
           assert_response :unprocessable_entity
-          assert_tag :errors, :child => {:tag => 'error', :content => 'relation_type is not included in the list'}
+          assert_tag :errors, :child => {:tag => 'error', :content => /relation_type is not included in the list/}
         end
       end
     end
@@ -99,6 +99,7 @@ class ApiTest::IssueRelationsTest < ActionController::IntegrationTest
         end
 
         assert_response :ok
+        assert_equal '', @response.body
         assert_nil IssueRelation.find_by_id(2)
       end
     end
