@@ -57,6 +57,12 @@ class CustomFieldTest < ActiveSupport::TestCase
     assert field.valid?
   end
 
+  def test_should_not_change_field_format_of_existing_custom_field
+    field = CustomField.find(1)
+    field.field_format = 'int'
+    assert_equal 'list', field.field_format
+  end
+
   def test_possible_values_should_accept_an_array
     field = CustomField.new
     field.possible_values = ["One value", ""]
@@ -75,6 +81,17 @@ class CustomFieldTest < ActiveSupport::TestCase
     assert_equal ["One value", "And another one"], field.possible_values
   end
 
+  if "string".respond_to?(:encoding)
+    def test_possible_values_stored_as_binary_should_be_utf8_encoded
+      field = CustomField.find(11)
+      assert_kind_of Array, field.possible_values
+      assert field.possible_values.size > 0
+      field.possible_values.each do |value|
+        assert_equal "UTF-8", value.encoding.name
+      end
+    end
+  end
+
   def test_destroy
     field = CustomField.find(1)
     assert field.destroy
@@ -191,4 +208,19 @@ class CustomFieldTest < ActiveSupport::TestCase
     assert f.valid_field_value?(['value1', 'value2'])
     assert !f.valid_field_value?(['value1', 'abc'])
   end
+
+  def test_value_class_should_return_the_class_used_for_fields_values
+    assert_equal User, CustomField.new(:field_format => 'user').value_class
+    assert_equal Version, CustomField.new(:field_format => 'version').value_class
+  end
+
+  def test_value_class_should_return_nil_for_other_fields
+    assert_nil CustomField.new(:field_format => 'text').value_class
+    assert_nil CustomField.new.value_class
+  end
+
+  def test_value_from_keyword_for_list_custom_field
+    field = CustomField.find(1)
+    assert_equal 'PostgreSQL', field.value_from_keyword('postgresql', Issue.find(1))
+  end
 end
