@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -84,25 +84,54 @@ class Redmine::PluginTest < ActiveSupport::TestCase
   end
 
   def test_requires_redmine
-    test = self
-    version = Redmine::VERSION.to_a.slice(0,3).join('.')
+    plugin = Redmine::Plugin.register(:foo) {}
+    Redmine::VERSION.stubs(:to_a).returns([2, 1, 3, "stable", 10817])
 
-    @klass.register :foo do
-      test.assert requires_redmine(:version_or_higher => '0.1.0')
-      test.assert requires_redmine(:version_or_higher => version)
-      test.assert requires_redmine(version)
-      test.assert_raise Redmine::PluginRequirementError do
-        requires_redmine(:version_or_higher => '99.0.0')
-      end
+    # Specific version without hash
+    assert plugin.requires_redmine('2.1.3')
+    assert plugin.requires_redmine('2.1')
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine('2.1.4')
+    end
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine('2.2')
+    end
 
-      test.assert requires_redmine(:version => version)
-      test.assert requires_redmine(:version => [version, '99.0.0'])
-      test.assert_raise Redmine::PluginRequirementError do
-        requires_redmine(:version => '99.0.0')
-      end
-      test.assert_raise Redmine::PluginRequirementError do
-        requires_redmine(:version => ['98.0.0', '99.0.0'])
-      end
+    # Specific version
+    assert plugin.requires_redmine(:version => '2.1.3')
+    assert plugin.requires_redmine(:version => ['2.1.3', '2.2.0'])
+    assert plugin.requires_redmine(:version => '2.1')
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine(:version => '2.2.0')
+    end
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine(:version => ['2.1.4', '2.2.0'])
+    end
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine(:version => '2.2')
+    end
+
+    # Version range
+    assert plugin.requires_redmine(:version => '2.0.0'..'2.2.4')
+    assert plugin.requires_redmine(:version => '2.1.3'..'2.2.4')
+    assert plugin.requires_redmine(:version => '2.0.0'..'2.1.3')
+    assert plugin.requires_redmine(:version => '2.0'..'2.2')
+    assert plugin.requires_redmine(:version => '2.1'..'2.2')
+    assert plugin.requires_redmine(:version => '2.0'..'2.1')
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine(:version => '2.1.4'..'2.2.4')
+    end
+
+
+    # Version or higher
+    assert plugin.requires_redmine(:version_or_higher => '0.1.0')
+    assert plugin.requires_redmine(:version_or_higher => '2.1.3')
+    assert plugin.requires_redmine(:version_or_higher => '2.1')
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine(:version_or_higher => '2.2.0')
+    end
+    assert_raise Redmine::PluginRequirementError do
+      plugin.requires_redmine(:version_or_higher => '2.2')
     end
   end
 
