@@ -1,3 +1,20 @@
+# Redmine - project management software
+# Copyright (C) 2006-2012  Jean-Philippe Lang
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
 require File.expand_path('../../../../../../test_helper', __FILE__)
 begin
   require 'mocha'
@@ -35,6 +52,13 @@ begin
         assert_nil @adapter.cat('doc-mkdir.txt', '12345678')
       end
 
+      def test_diff
+        diff1 = @adapter.diff('doc-mkdir.txt', 3, 2)
+        assert_equal 21, diff1.size
+        buf =  diff1[14].gsub(/\r\n|\r|\n/, "")
+        assert_equal "-Display more information.", buf
+      end
+
       def test_diff_path_invalid
         assert_equal [], @adapter.diff('invalid', 1)
       end
@@ -127,6 +151,22 @@ begin
         assert_nil @adapter.entries(nil, 12345678)
       end
 
+      def test_revisions
+        revisions = @adapter.revisions(nil, 4, 2)
+        assert_equal 3, revisions.size
+        assert_equal 2, revisions[2].identifier
+        assert_equal 'jsmith@foo.bar-20071203175224-v0eog5d5wrgdrshg', revisions[2].scmid
+        assert_equal 4, revisions[0].identifier
+        assert_equal 'jsmith@foo.bar-20071203175422-t40bf8li5zz0c4cg', revisions[0].scmid
+        assert_equal 2, revisions[0].paths.size
+        assert_equal 'D', revisions[0].paths[0][:action]
+        assert_equal '/doc-deleted.txt', revisions[0].paths[0][:path]
+        assert_equal 'docdeleted.txt-20071203175320-iwwj561ojuubs3gt-1', revisions[0].paths[0][:revision]
+        assert_equal 'M', revisions[0].paths[1][:action]
+        assert_equal '/directory/doc-ls.txt', revisions[0].paths[1][:path]
+        assert_equal 'docls.txt-20071203175005-a3hyc3mn0shl7cgu-1', revisions[0].paths[1][:revision]
+      end
+
       def test_revisions_path_invalid
         assert_nil @adapter.revisions('invalid')
       end
@@ -136,6 +176,37 @@ begin
         assert_nil @adapter.revisions(nil, 12345678, 87654321)
       end
 
+      def test_entry
+        entry = @adapter.entry()
+        assert_equal "", entry.path
+        assert_equal "dir", entry.kind
+        entry = @adapter.entry('')
+        assert_equal "", entry.path
+        assert_equal "dir", entry.kind
+        assert_nil @adapter.entry('invalid')
+        assert_nil @adapter.entry('/invalid')
+        assert_nil @adapter.entry('/invalid/')
+        assert_nil @adapter.entry('invalid/invalid')
+        assert_nil @adapter.entry('invalid/invalid/')
+        assert_nil @adapter.entry('/invalid/invalid')
+        assert_nil @adapter.entry('/invalid/invalid/')
+        ["doc-ls.txt", "/doc-ls.txt"].each do |path|
+          entry = @adapter.entry(path, 2)
+          assert_equal "doc-ls.txt", entry.path
+          assert_equal "file", entry.kind
+        end
+        ["directory", "/directory", "/directory/"].each do |path|
+          entry = @adapter.entry(path, 2)
+          assert_equal "directory", entry.path
+          assert_equal "dir", entry.kind
+        end
+        ["directory/document.txt", "/directory/document.txt"].each do |path|
+          entry = @adapter.entry(path, 2)
+          assert_equal "directory/document.txt", entry.path
+          assert_equal "file", entry.kind
+        end
+      end
+
       private
 
       def test_scm_version_for(scm_command_version, version)
