@@ -1,4 +1,19 @@
-# encoding: utf-8
+# Redmine - project management software
+# Copyright (C) 2006-2012  Jean-Philippe Lang
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 require File.expand_path('../../../../../../test_helper', __FILE__)
 begin
@@ -10,16 +25,6 @@ begin
     FELIX_HEX  = "Felix Sch\xC3\xA4fer"
     CHAR_1_HEX = "\xc3\x9c"
 
-    ## Ruby uses ANSI api to fork a process on Windows.
-    ## Japanese Shift_JIS and Traditional Chinese Big5 have 0x5c(backslash) problem
-    ## and these are incompatible with ASCII.
-    ## Git for Windows (msysGit) changed internal API from ANSI to Unicode in 1.7.10
-    ## http://code.google.com/p/msysgit/issues/detail?id=80
-    ## So, Latin-1 path tests fail on Japanese Windows
-    WINDOWS_PASS = (Redmine::Platform.mswin? &&
-                       Redmine::Scm::Adapters::GitAdapter.client_version_above?([1, 7, 10]))
-    WINDOWS_SKIP_STR = "TODO: This test fails in Git for Windows above 1.7.10"
-
     ## Git, Mercurial and CVS path encodings are binary.
     ## Subversion supports URL encoding for path.
     ## Redmine Mercurial adapter and extension use URL encoding.
@@ -29,6 +34,16 @@ begin
     JRUBY_SKIP_STR = "TODO: This test fails in JRuby"
 
     if File.directory?(REPOSITORY_PATH)
+      ## Ruby uses ANSI api to fork a process on Windows.
+      ## Japanese Shift_JIS and Traditional Chinese Big5 have 0x5c(backslash) problem
+      ## and these are incompatible with ASCII.
+      ## Git for Windows (msysGit) changed internal API from ANSI to Unicode in 1.7.10
+      ## http://code.google.com/p/msysgit/issues/detail?id=80
+      ## So, Latin-1 path tests fail on Japanese Windows
+      WINDOWS_PASS = (Redmine::Platform.mswin? &&
+                      Redmine::Scm::Adapters::GitAdapter.client_version_above?([1, 7, 10]))
+      WINDOWS_SKIP_STR = "TODO: This test fails in Git for Windows above 1.7.10"
+
       def setup
         adapter_class = Redmine::Scm::Adapters::GitAdapter
         assert adapter_class
@@ -384,7 +399,7 @@ begin
         str_felix_hex  = FELIX_HEX.dup
         last_rev_author = last_rev.author
         if last_rev_author.respond_to?(:force_encoding)
-          last_rev_author.force_encoding('UTF-8')
+          str_felix_hex.force_encoding('ASCII-8BIT')
         end
         assert_equal "ed5bb786bbda2dee66a2d50faf51429dbc043a7b", last_rev.scmid
         assert_equal "ed5bb786bbda2dee66a2d50faf51429dbc043a7b", last_rev.identifier
@@ -445,6 +460,23 @@ begin
         assert_equal Time.gm(2009, 6, 19, 4, 37, 23), readme.lastrev.time
       end
 
+      def test_entries_wrong_path_encoding
+        adpt = Redmine::Scm::Adapters::GitAdapter.new(
+                      REPOSITORY_PATH,
+                      nil,
+                      nil,
+                      nil,
+                      'EUC-JP'
+                   )
+        entries1 = adpt.entries('latin-1-dir', '64f1f3e8')
+        assert entries1
+        assert_equal 3, entries1.size
+        f1 = entries1[1]
+        assert_equal nil, f1.name
+        assert_equal nil, f1.path
+        assert_equal 'file', f1.kind
+      end
+
       def test_entries_latin_1_files
         entries1 = @adapter.entries('latin-1-dir', '64f1f3e8')
         assert entries1
@@ -472,6 +504,37 @@ begin
         end
       end
 
+      def test_entry
+        entry = @adapter.entry()
+        assert_equal "", entry.path
+        assert_equal "dir", entry.kind
+        entry = @adapter.entry('')
+        assert_equal "", entry.path
+        assert_equal "dir", entry.kind
+        assert_nil @adapter.entry('invalid')
+        assert_nil @adapter.entry('/invalid')
+        assert_nil @adapter.entry('/invalid/')
+        assert_nil @adapter.entry('invalid/invalid')
+        assert_nil @adapter.entry('invalid/invalid/')
+        assert_nil @adapter.entry('/invalid/invalid')
+        assert_nil @adapter.entry('/invalid/invalid/')
+        ["README", "/README"].each do |path|
+          entry = @adapter.entry(path, '7234cb2750b63f')
+          assert_equal "README", entry.path
+          assert_equal "file", entry.kind
+        end
+        ["sources", "/sources", "/sources/"].each do |path|
+          entry = @adapter.entry(path, '7234cb2750b63f')
+          assert_equal "sources", entry.path
+          assert_equal "dir", entry.kind
+        end
+        ["sources/watchers_controller.rb", "/sources/watchers_controller.rb"].each do |path|
+          entry = @adapter.entry(path, '7234cb2750b63f')
+          assert_equal "sources/watchers_controller.rb", entry.path
+          assert_equal "file", entry.kind
+        end
+      end
+
       def test_path_encoding_default_utf8
         adpt1 = Redmine::Scm::Adapters::GitAdapter.new(
                                   REPOSITORY_PATH
@@ -493,7 +556,7 @@ begin
 
       def test_cat_revision_invalid
         assert     @adapter.cat('README')
-        assert_nil @adapter.cat('README', 'abcd1234efgh')
+        assert_nil @adapter.cat('README', '1234abcd5678')
       end
 
       def test_diff_path_invalid
@@ -501,9 +564,9 @@ begin
       end
 
       def test_diff_revision_invalid
-        assert_nil @adapter.diff(nil, 'abcd1234efgh')
-        assert_nil @adapter.diff(nil, '713f4944648826f5', 'abcd1234efgh')
-        assert_nil @adapter.diff(nil, 'abcd1234efgh', '713f4944648826f5')
+        assert_nil @adapter.diff(nil, '1234abcd5678')
+        assert_nil @adapter.diff(nil, '713f4944648826f5', '1234abcd5678')
+        assert_nil @adapter.diff(nil, '1234abcd5678', '713f4944648826f5')
       end
 
       def test_annotate_path_invalid
@@ -512,7 +575,7 @@ begin
 
       def test_annotate_revision_invalid
         assert     @adapter.annotate('README')
-        assert_nil @adapter.annotate('README', 'abcd1234efgh')
+        assert_nil @adapter.annotate('README', '1234abcd5678')
       end
 
       private
