@@ -1,5 +1,5 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -97,10 +97,52 @@ class RepositoryTest < ActiveSupport::TestCase
     assert_equal [repository1, repository2], Project.find(3).repositories.sort
   end
 
+  def test_identifier_should_accept_letters_digits_dashes_and_underscores
+    r = Repository::Subversion.new(
+      :project_id => 3,
+      :identifier => 'svn-123_45',
+      :url => 'file:///svn'
+    )
+    assert r.save
+  end
+  
+  def test_identifier_should_not_be_frozen_for_a_new_repository
+    assert_equal false, Repository.new.identifier_frozen?
+  end
+
+  def test_identifier_should_not_be_frozen_for_a_saved_repository_with_blank_identifier
+    Repository.update_all(["identifier = ''"], "id = 10")
+
+    assert_equal false, Repository.find(10).identifier_frozen?
+  end
+
+  def test_identifier_should_be_frozen_for_a_saved_repository_with_valid_identifier
+    Repository.update_all(["identifier = 'abc123'"], "id = 10")
+
+    assert_equal true, Repository.find(10).identifier_frozen?
+  end
+
+  def test_identifier_should_not_accept_change_if_frozen
+    r = Repository.new(:identifier => 'foo')
+    r.stubs(:identifier_frozen?).returns(true)
+
+    r.identifier = 'bar'
+    assert_equal 'foo', r.identifier
+  end
+
+  def test_identifier_should_accept_change_if_not_frozen
+    r = Repository.new(:identifier => 'foo')
+    r.stubs(:identifier_frozen?).returns(false)
+
+    r.identifier = 'bar'
+    assert_equal 'bar', r.identifier
+  end
+
   def test_destroy
-    changesets = Changeset.count(:all, :conditions => "repository_id = 10")
-    changes = Change.count(:all, :conditions => "repository_id = 10",
-                           :joins => :changeset)
+    repository = Repository.find(10)
+    changesets = repository.changesets.count
+    changes = repository.filechanges.count
+
     assert_difference 'Changeset.count', -changesets do
       assert_difference 'Change.count', -changes do
         Repository.find(10).destroy
@@ -139,7 +181,6 @@ class RepositoryTest < ActiveSupport::TestCase
 
   def test_scan_changesets_for_issue_ids
     Setting.default_language = 'en'
-    Setting.notified_events = ['issue_added','issue_updated']
 
     # choosing a status to apply to fix issues
     Setting.commit_fix_status_id = IssueStatus.find(
@@ -156,7 +197,9 @@ class RepositoryTest < ActiveSupport::TestCase
     assert !fixed_issue.status.is_closed?
     old_status = fixed_issue.status
 
-    Repository.scan_changesets_for_issue_ids
+    with_settings :notified_events => %w(issue_added issue_updated) do
+      Repository.scan_changesets_for_issue_ids
+    end
     assert_equal [101, 102], Issue.find(3).changeset_ids
 
     # fixed issues
