@@ -1,16 +1,16 @@
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
@@ -18,17 +18,47 @@
 require File.expand_path('../../test_helper', __FILE__)
 
 class RoleTest < ActiveSupport::TestCase
-  fixtures :roles, :workflows
+  fixtures :roles, :workflows, :trackers
+
+  def test_sorted_scope
+    assert_equal Role.all.sort, Role.sorted.all
+  end
+
+  def test_givable_scope
+    assert_equal Role.all.reject(&:builtin?).sort, Role.givable.all
+  end
+
+  def test_builtin_scope
+    assert_equal Role.all.select(&:builtin?).sort, Role.builtin(true).all.sort
+    assert_equal Role.all.reject(&:builtin?).sort, Role.builtin(false).all.sort
+  end
+
+  def test_copy_from
+    role = Role.find(1)
+    copy = Role.new.copy_from(role)
+
+    assert_nil copy.id
+    assert_equal '', copy.name
+    assert_equal role.permissions, copy.permissions
+
+    copy.name = 'Copy'
+    assert copy.save
+  end
 
   def test_copy_workflows
     source = Role.find(1)
-    assert_equal 90, source.workflows.size
-    
+    assert_equal 90, source.workflow_rules.size
+
     target = Role.new(:name => 'Target')
     assert target.save
-    target.workflows.copy(source)
+    target.workflow_rules.copy(source)
     target.reload
-    assert_equal 90, target.workflows.size
+    assert_equal 90, target.workflow_rules.size
+  end
+
+  def test_permissions_should_be_unserialized_with_its_coder
+    Role::PermissionsAttributeCoder.expects(:load).once
+    Role.find(1).permissions
   end
 
   def test_add_permission
@@ -49,7 +79,7 @@ class RoleTest < ActiveSupport::TestCase
     assert ! role.permissions.include?(perm[0])
     assert_equal size - 2, role.permissions.size
   end
-  
+
   def test_name
     I18n.locale = 'fr'
     assert_equal 'Manager', Role.find(1).name
@@ -57,6 +87,10 @@ class RoleTest < ActiveSupport::TestCase
     assert_equal 'Non membre', Role.non_member.name
   end
 
+  def test_find_all_givable
+    assert_equal Role.all.reject(&:builtin?).sort, Role.find_all_givable
+  end
+
   context "#anonymous" do
     should "return the anonymous role" do
       role = Role.anonymous
